#ifndef UTIL___UTIL_MISC__HPP
#define UTIL___UTIL_MISC__HPP

/*  $Id: util_misc.hpp,v 1.1.1.1 2010/05/19 18:36:00 rhubley Exp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Author:  Sergey Satskiy
 *
 */

#include <corelib/ncbistd.hpp>


BEGIN_NCBI_SCOPE


/// Get a password without echoing the user input
///
/// @param
///   prompt Printed before the user input
/// @return
///   The user input till the first carriage return
/// @warning
///   UNIX and Windows implementations differ at processing Ctrl+C:
///   - UNIX: The usual SIGINT is generated by the system
///   - Windows: an exception is thrown
/// @exception
///   The function throws CGetPasswordFromConsoleException exception if
///   anything goes wrong
NCBI_XUTIL_EXPORT
extern string g_GetPasswordFromConsole(const string& prompt);



/// Exception class for g_GetPasswordFromConsole
class NCBI_XUTIL_EXPORT  CGetPasswordFromConsoleException : public CException
{
public:
    enum EErrCode {
        eGetTerminalAttrError,  ///< UNIX specific: cannot get term attributes
        eSetTerminalAttrError,  ///< UNIX specific: cannot set term attributes
        eGetLineError,          ///< UNIX specific: error reading from terminal
        eKeyboardInterrupt      ///< WIN specific:  Ctrl+C has been sent
    };

    /// Translate from the error code value to its string representation
    virtual const char* GetErrCodeString(void) const;

    NCBI_EXCEPTION_DEFAULT(CGetPasswordFromConsoleException, CException);
};


END_NCBI_SCOPE


#endif  /* UTIL___UTIL_MISC__HPP */
